@@ -69,7 +69,7 @@ func TestFormatNilLocationEmptyBody(t *testing.T) {
 func TestFormatNilLocationFunctionArgs(t *testing.T) {
 	b := ast.NewBody()
 	s := ast.StringTerm(" ")
-	s.SetLocation(location.NewLocation([]byte("foo"), "p.rego", 2, 2))
+	s.SetLocation(location.NewLocation([]byte("\" \""), "p.rego", 2, 2))
 	b.Append(ast.Split.Expr(ast.NewTerm(ast.Var("__local1__")), s, ast.NewTerm(ast.Var("__local2__"))))
 	exp := "split(__local1__, \" \", __local2__)\n"
 	bs, err := Ast(b)
@@ -293,7 +293,7 @@ func TestFormatAST(t *testing.T) {
 	cases := []struct {
 		note        string
 		regoVersion ast.RegoVersion
-		toFmt       interface{}
+		toFmt       any
 		expected    string
 	}{
 		{
@@ -309,6 +309,14 @@ func TestFormatAST(t *testing.T) {
 			},
 			expected: `"foo"`,
 		},
+		{
+			note: "string, raw",
+			toFmt: &ast.Term{
+				Value:    ast.String(`foo`),
+				Location: &ast.Location{Text: []byte("`foo`")},
+			},
+			expected: "`foo`",
+		},
 		{
 			note:     "var wildcard",
 			toFmt:    ast.Var(`$12`),
@@ -638,6 +646,83 @@ a[_x[y][[z, w]]]`,
 			}(),
 			expected: `concat(",", [__local1__], __local0__)`,
 		},
+		{
+			note: "template-string",
+			toFmt: ast.TemplateStringTerm(false,
+				ast.StringTerm("foo "),
+				&ast.Expr{
+					Terms: ast.VarTerm("x"),
+				},
+				ast.StringTerm(" "),
+				&ast.Expr{
+					Terms: ast.RefTerm(ast.VarTerm("input"), ast.StringTerm("y")),
+				},
+				ast.StringTerm(" \n "),
+				&ast.Expr{
+					Terms: ast.Round.Call(ast.NumberTerm("1.2")),
+				},
+			),
+			expected: `$"foo {x} {input.y} \n {round(1.2)}"`,
+		},
+		{
+			note: "template-string, multi-line",
+			toFmt: ast.TemplateStringTerm(true,
+				ast.StringTerm(`foo `),
+				&ast.Expr{
+					Terms: ast.VarTerm("x"),
+				},
+				ast.StringTerm(` `),
+				&ast.Expr{
+					Terms: ast.RefTerm(ast.VarTerm("input"), ast.StringTerm("y")),
+				},
+				ast.StringTerm(` 
+ `),
+				&ast.Expr{
+					Terms: ast.Round.Call(ast.NumberTerm("1.2")),
+				},
+			),
+			expected: "$`foo {x} {input.y} \n {round(1.2)}`",
+		},
+		{
+			note: "template-string, single-line, string template-expressions",
+			toFmt: ast.TemplateStringTerm(false,
+				ast.StringTerm("foo "),
+				&ast.Expr{
+					Terms: &ast.Term{
+						Value:    ast.String("bar"),
+						Location: &ast.Location{Text: []byte(`"bar"`)},
+					},
+				},
+				ast.StringTerm(" "),
+				&ast.Expr{
+					Terms: &ast.Term{
+						Value:    ast.String(`baz`),
+						Location: &ast.Location{Text: []byte("`baz`")},
+					},
+				},
+			),
+			expected: `$"foo {"bar"} {` + "`baz`" + `}"`,
+		},
+		{
+			note: "template-string, multi-line, string template-expressions",
+			toFmt: ast.TemplateStringTerm(true,
+				ast.StringTerm("foo "),
+				&ast.Expr{
+					Terms: &ast.Term{
+						Value:    ast.String("bar"),
+						Location: &ast.Location{Text: []byte(`"bar"`)},
+					},
+				},
+				ast.StringTerm(" "),
+				&ast.Expr{
+					Terms: &ast.Term{
+						Value:    ast.String(`baz`),
+						Location: &ast.Location{Text: []byte("`baz`")},
+					},
+				},
+			),
+			expected: "$`foo {\"bar\"} {`baz`}`",
+		},
 	}
 
 	for _, tc := range cases {
@@ -671,6 +756,45 @@ a[_x[y][[z, w]]]`,
 	}
 }
 
+func TestFormatAST_Error(t *testing.T) {
+	cases := []struct {
+		note        string
+		regoVersion ast.RegoVersion
+		toFmt       any
+		expErr      string
+	}{
+		{
+			note:   "package with only data term",
+			toFmt:  &ast.Package{Path: ast.Ref{ast.DefaultRootDocument}},
+			expErr: `rego_format_error: invalid package path: data`,
+		},
+		{
+			note: "module with package with only data term",
+			toFmt: &ast.Module{
+				Package: &ast.Package{Path: ast.Ref{ast.DefaultRootDocument}},
+			},
+			expErr: `rego_format_error: invalid package path: data`,
+		},
+	}
+
+	for _, tc := range cases {
+		t.Run(tc.note, func(t *testing.T) {
+			_, err := AstWithOpts(tc.toFmt, Opts{
+				RegoVersion: tc.regoVersion,
+				ParserOptions: &ast.ParserOptions{
+					RegoVersion: tc.regoVersion,
+				},
+			})
+			if err == nil {
+				t.Fatalf("Expected error, got nil")
+			}
+			if !strings.Contains(err.Error(), tc.expErr) {
+				t.Fatalf("Expected error to contain:\n\n%q\n\ngot:\n\n%q", tc.expErr, err.Error())
+			}
+		})
+	}
+}
+
 func TestFormatDeepCopy(t *testing.T) {
 
 	original := ast.Body{
@@ -701,10 +825,7 @@ func differsAt(a, b []byte) (int, int) {
 	if bytes.Equal(a, b) {
 		return 0, 0
 	}
-	minLen := len(a)
-	if minLen > len(b) {
-		minLen = len(b)
-	}
+	minLen := min(len(a), len(b))
 	ln := 1
 	for i := range minLen {
 		if a[i] == '\n' {
@@ -868,24 +989,126 @@ p contains x if {
 	}
 }
 
-// 382	   3064960 ns/op	 4573131 B/op	   26266 allocs/op // no optimizations
-// 685	   1737719 ns/op	 1972193 B/op	   14160 allocs/op // pre-allocate partitionComments
-// 708	   1674343 ns/op	 1916700 B/op	   11556 allocs/op // static memberRef & memberWithKeyRef
-// 746	   1594546 ns/op	 1882652 B/op	   10644 allocs/op // various minor fixes
-// 1250	    853508 ns/op	  441730 B/op	    8895 allocs/op // partitionComments early return if unchanged
-// 1396	    812859 ns/op	  362651 B/op	    8811 allocs/op // partitionComments reuse backing array
+func TestGroupableOneLinerRules(t *testing.T) {
+	contents := []byte(`package test
+
+foo := 1 if input.x
+foo := 2 if not input.x
+
+a := 1
+b := 2
+
+c := 3
+
+d := 4
+
+# comment above group
+e := 5
+f := 6
+`)
+
+	formatted, err := Source("test.rego", contents)
+	if err != nil {
+		t.Fatalf("unexpected error: %v", err)
+	}
+
+	if !bytes.Equal(formatted, contents) {
+		t.Fatalf("expected %q but got %q", formatted, contents)
+	}
+}
+
+func TestFormatKeywordsInRefs(t *testing.T) {
+	regoVersions := map[string]ast.RegoVersion{
+		"v0": ast.RegoV0,
+		"v1": ast.RegoV1,
+	}
+
+	for versionName, regoVersion := range regoVersions {
+		t.Run(versionName, func(t *testing.T) {
+			regoFiles, err := filepath.Glob(fmt.Sprintf("testfiles/%s/*.rego.formatted_no_keywords_in_refs", versionName))
+			if err != nil {
+				panic(err)
+			}
+
+			for _, expected_rego := range regoFiles {
+				original_rego := strings.TrimSuffix(expected_rego, ".formatted_no_keywords_in_refs")
+				t.Run(original_rego, func(t *testing.T) {
+					expected, err := os.ReadFile(expected_rego)
+					if err != nil {
+						t.Fatalf("Failed to read expected rego source: %v", err)
+					}
+
+					original, err := os.ReadFile(original_rego)
+					if err != nil {
+						t.Fatalf("Failed to read rego source: %v", err)
+					}
+
+					caps := ast.CapabilitiesForThisVersion(ast.CapabilitiesRegoVersion(regoVersion))
+					feats := []string{}
+					for _, f := range caps.Features {
+						if f != ast.FeatureKeywordsInRefs {
+							feats = append(feats, f)
+						}
+					}
+					caps.Features = feats
+
+					popts := ast.ParserOptions{
+						RegoVersion: regoVersion,
+					}
+					opts := Opts{
+						RegoVersion:   regoVersion,
+						ParserOptions: &popts,
+						Capabilities:  caps,
+					}
+
+					formatted, err := SourceWithOpts(original_rego, original, opts)
+					if err != nil {
+						t.Fatalf("Failed to format file: %v", err)
+					}
+
+					if ln, at := differsAt(formatted, expected); ln != 0 {
+						t.Fatalf("Expected formatted bytes to equal expected bytes but differed near line %d / byte %d (got: %q, expected: %q):\n%s", ln, at, formatted[at], expected[at], prefixWithLineNumbers(formatted))
+					}
+
+					if _, err := ast.ParseModuleWithOpts(original_rego+".tmp", string(formatted), popts); err != nil {
+						t.Fatalf("Failed to parse formatted bytes: %v", err)
+					}
+
+					formatted, err = SourceWithOpts(original_rego, formatted, opts)
+					if err != nil {
+						t.Fatalf("Failed to double format file")
+					}
+
+					if ln, at := differsAt(formatted, expected); ln != 0 {
+						t.Fatalf("Expected roundtripped bytes to equal expected bytes but differed near line %d / byte %d:\n%s", ln, at, prefixWithLineNumbers(formatted))
+					}
+
+				})
+			}
+		})
+	}
+}
+
+// 3064960 ns/op	 4573131 B/op	   26266 allocs/op // no optimizations
+// 1737719 ns/op	 1972193 B/op	   14160 allocs/op // pre-allocate partitionComments
+// 1674343 ns/op	 1916700 B/op	   11556 allocs/op // static memberRef & memberWithKeyRef
+// 1594546 ns/op	 1882652 B/op	   10644 allocs/op // various minor fixes
+// _853508 ns/op	  441730 B/op	    8895 allocs/op // partitionComments early return if unchanged
+// _812859 ns/op	  362651 B/op	    8811 allocs/op // partitionComments reuse backing array
+// _676179 ns/op	  995204 B/op	    8850 allocs/op // regression in 0fb7526
+// _513570 ns/op	  378787 B/op	    8775 allocs/op // addressed regression with sync.Pool
+// _481681 ns/op	  352130 B/op	    7954 allocs/op // new util.NewSlicePool using only pointers
+// _365116 ns/op	  160528 B/op	    2098 allocs/op // new SkipDefensiveCopying option
 func BenchmarkFormatLargePolicy(b *testing.B) {
 	contents, err := os.ReadFile("testdata/bench.rego")
 	if err != nil {
 		b.Fatalf("Failed to read rego source: %v", err)
 	}
 	module := ast.MustParseModule(string(contents))
+	opts := Opts{RegoVersion: ast.RegoV1, SkipDefensiveCopying: true}
 
-	b.ResetTimer()
-
-	for range b.N {
-		_, err := AstWithOpts(module, Opts{RegoVersion: ast.RegoV1})
-		if err != nil {
+	for b.Loop() {
+		if _, err := AstWithOpts(module, opts); err != nil {
 			b.Fatal(err)
 		}
 	}
